// 5648-C84, 5648-C83, (C) Copyright IBM Corporation, 1997, 2000
// All rights reserved. Licensed Materials Property of IBM
// Note to US Government users: Documentation related to restricted rights
// Use, duplication or disclosure is subject to restrictions set forth in GSA ADP Schedule with IBM Corp.
// This page may contain other proprietary notices and copyright information, the terms of which must be observed and followed.
//
// This program may be used, executed, copied, modified and distributed
// without royalty for the purpose of developing, using,
// marketing, or distributing.
//
// @(#) 1.6 src/ejb/java/com/ibm/ejs/doc/accountBM/AccountBMBean.java, ejbdemo, boss 5/21/99 14:41:34 [5/25/99 15:38:02]
package com.ibm.ejs.doc.accountBM;

/**
 * The bean class of the AccountBM entity bean.
 **/
import java.rmi.RemoteException;
import java.util.*;
import javax.ejb.*;
import java.lang.*;
import java.sql.*;
import com.ibm.ejs.doc.account.InsufficientFundsException;

public class AccountBMBean implements javax.ejb.EntityBean {

   private EntityContext entityContext = null;

   /**
     Property used to name the database to persist to.  The DBURL
     environment property is set at deploy/install time.
     Example: "jdbc:odbc:ejbdemo".
     */
   private static final String DBURLProp      = "DBURL";

   /**
     Property used to name the database driver.  The DriverName
     environment property is set at deploy/install time.
     Example: "sun.jdbc.odbc.JdbcOdbcDriver", or
     "COM.ibm.db2.jdbc.app.DB2Driver".
     */
   private static final String DriverNameProp = "DriverName";

   private static final String DBLoginProp    = "DBLogin";
   private static final String DBPasswordProp = "DBPassword";
   private static final String TableNameProp = "TableName";

   private String jdbcUrl, driverName, DBLogin, DBPassword, tableName;

   private long accountId = 0;
   private int type = 1;
   private float balance = 0.0f;

   private Connection jdbcConn = null;


   // static data
   private static transient ListResourceBundle msgs = null;

   /**
    * The default constructor for creating AccountBMBean bean instances.
    */
   public AccountBMBean() {

   }
   /**
    * Adds the specified amount to the account balance and returns the new balance.
    * @param amount float The amount to add to the account balance.
    * @return float The new balance in the account after the method executes.
    */
   public float add(float amount) {
      balance += amount;
      return balance;
   }
   /**
    * Checks to ensure that a valid connection to the data source is available. If the
    * connection is not valid, the method calls the makeConnection method to create
    * a connection.
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    */
   private void checkConnection() throws RemoteException {
      if (jdbcConn == null) {
         makeConnection();
      }

      return;
   }
   /**
    * Disposes of a connection to a data source by committing any outstanding transactions,
    * closing the data source connection, and setting the connection reference to null.
    */
   private void dropConnection() {
      try {
         // Close and delete jdbcConn
         jdbcConn.commit();
      }
      catch (Exception e) {
         // Could not commit
         System.out.println(getMsg("cantCommit"));
         e.printStackTrace();
      }
      finally {
         try {
            jdbcConn.close();
         }
         catch (Exception e) {
            // Could not drop connection
            System.out.println(getMsg("dropFailed"));
            e.printStackTrace();
         }
         jdbcConn = null;
      }

   }
   /**
    * Activates the bean instance by getting and setting the accountId variable and
    * ensuring that a valid connection to the data source exists.
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    */
   public void ejbActivate() throws RemoteException {
      AccountBMKey accountKey = (AccountBMKey) entityContext.getPrimaryKey();
      accountId = accountKey.accountId;

      checkConnection();
   }
   /**
    * Creates the bean instance for the accountId encapsulated in the AccountBMKey object
    * by invoking the ejbCreate method with AccountBMKey, int, and float parameters 
    * by using default values for the account type (1 for savings) and the initial balance (0).
    * @param key com.ibm.ejs.doc.account.AccountBMKey The AccountBMKey object that encapsulates the
    * account Id used to create the specified account.
    * @return key com.ibm.ejs.doc.account.AccountBMKey The AccountBMKey object that encapsulates the
    * account Id used to create the specified account.
    * @exception javax.ejb.CreateException The AccountBM bean instance was not created.
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    */
   public AccountBMKey ejbCreate(AccountBMKey key) throws CreateException, RemoteException { 
      return ejbCreate(key, 1, 0.0f); 
   }
   /**
    * Creates the bean instance for the account Id encapsulated in the AccountBMKey object.
    * @param key com.ibm.ejs.doc.accountBM.AccountBMKey The AccountBMKey object that encapsulates the
    * account Id used to create the specified account.
    * @param type int The type of the account to be created: 1 for savings, 2 for checking.
    * @param initialBalance float The initial balance contained in the account.
    * @return key com.ibm.ejs.doc.accountBM.AccountBMKey The AccountBMKey object that encapsulates the
    * account Id used to create the specified account.
    * @exception javax.ejb.CreateException The AccountBM bean instance was not created.
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    */
   public AccountBMKey ejbCreate(AccountBMKey key, int type, float balance) 
   throws CreateException, RemoteException {

      accountId = key.accountId;
      this.type = type;
      this.balance = balance;

      makeConnection();

      // INSERT into the database
      try {
         String sqlString = "INSERT INTO " + tableName + 
                            " (balance, type, accountid) VALUES (?,?,?)";
         PreparedStatement sqlStatement = jdbcConn.prepareStatement(sqlString);
         sqlStatement.setFloat(1, balance);
         sqlStatement.setInt(2, type);
         sqlStatement.setLong(3, accountId);

         // Execute query
         int updateResults = sqlStatement.executeUpdate();
         if (updateResults != 1) {
            // Either fewer than or more than one row was deleted
            throw(new CreateException(getMsg("errUpdate", Integer.toString(updateResults))));
         }
      }
      catch (Exception e) {
         // Error occurred during data update
         throw(new RemoteException(getMsg("errDB"), e));
      }
      return key;
   }
   /**
    * Determines if an account with the accountId encapsulated in the AccountBMKey object
    * exists. If only one account exists, the method returns the AccountBMKey object.
    * @param key com.ibm.ejs.doc.accountBM.AccountBMKey The AccountBMKey object that encapsulates
    * the accountId of the account for which the method is searching.
    * @return key com.ibm.ejs.doc.accountBM.AccountBMKey The AccountBMKey object that encapsulates
    * the accountId corresponding to the account found by the method.
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    * @exception javax.ejb.FinderException The specified accountId was not found.
    */
   public AccountBMKey ejbFindByPrimaryKey (AccountBMKey key)
   throws FinderException, RemoteException {

      boolean wasFound = false;
      boolean foundMultiples = false;

      makeConnection();

      try {
         // SELECT from database
         String sqlString = 
         "SELECT balance, type, accountid FROM " + tableName + " WHERE accountid = ?";
         PreparedStatement sqlStatement = jdbcConn.prepareStatement(sqlString);
         long keyValue = key.accountId;
         sqlStatement.setLong(1, keyValue);

         // Execute query
         ResultSet sqlResults = sqlStatement.executeQuery();

         // Advance cursor (even though there should be only one item)
         // wasFound is set to true if there is one
         wasFound = sqlResults.next();

         // foundMultiples will be true if more than one is found.
         foundMultiples = sqlResults.next();
      }
      catch (Exception e) {
         // Throw a resonable exception.
         throw(new RemoteException(getMsg("errDB"), e));
      }

      dropConnection();

      if (wasFound && !foundMultiples) {
         return key;
      }
      else {

         String foundStatus = null;
         if (!wasFound) {
            getMsg("keyNotFound", key.toString());
         }


         if (foundMultiples) {
            getMsg("tooManyKeys", key.toString());         
         }

         throw(new FinderException(foundStatus));
      }
   }
   /**
    * Finds all accounts with a balance greater than the specified amount.
    * @param amount float The balance above which an account must contain to be returned by this method.
    * @return java.util.Enumeration The collection of accounts that contain balances greater than
    * the specified amount.
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    */
   public Enumeration ejbFindLargeAccounts(float amount) throws RemoteException {
      makeConnection();

      Enumeration result;

      try {
         // SELECT from database
         String sqlString = "SELECT accountid FROM " + tableName + " WHERE balance >= ?";
         PreparedStatement sqlStatement = jdbcConn.prepareStatement(sqlString);
         sqlStatement.setFloat(1, amount);

         // Execute query
         ResultSet sqlResults = sqlStatement.executeQuery();

         // Set up Hashtable to contain list of primary keys
         Hashtable resultTable = new Hashtable();

         // Loop through result set until there are no more entries
         // Insert each primary key into the resultTable
         while (sqlResults.next() == true) {
            long acctId = sqlResults.getLong(1);
            Long memberId = new Long(acctId);
            AccountBMKey key = new AccountBMKey(acctId);
            resultTable.put(memberId, key);
         }

         // Return the resultTable as an Enumeration
         result = resultTable.elements();
         return result;
      }
      catch (Exception e) {
         // Throw a resonable exception.
         throw(new RemoteException(getMsg("errDB"), e));
      }
      finally {
         dropConnection();
      }
   }
   /**
    * Loads the data in the data source into the AccountBMBean class, overwriting the
    * the values of the persistent data variables.
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    */
   public void ejbLoad () throws RemoteException {
      // Get data from database
      try {
         AccountBMKey accountKey = (AccountBMKey) entityContext.getPrimaryKey();
         accountId = accountKey.accountId;
      }
      catch (Exception ex) {
         ex.toString();
      }

      checkConnection();

      try {
         // SELECT from database
         String sqlString = "SELECT balance, type, accountid FROM " + tableName + " WHERE accountid = ?";
         PreparedStatement sqlStatement = jdbcConn.prepareStatement(sqlString);
         sqlStatement.setLong(1, accountId);

         // Execute query
         ResultSet sqlResults = sqlStatement.executeQuery();

         // Advance cursor (even though there should be only one item)
         sqlResults.next();

         // Pull out results
         balance = sqlResults.getFloat(1);
         type = sqlResults.getInt(2);
      }
      catch (Exception e) {
         // Something happened while loading data.
         throw(new RemoteException(getMsg("errDB"), e));
      }
   }
   /**
    * Passivates the bean instance by dropping its connection to the data source.
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    */
   public void ejbPassivate() throws RemoteException {
      // Deactivate the bean and drop its database connections
      dropConnection();
   }
   /**
    * Removes the bean instance from the data source and drops the bean's connection to the data
    * source.
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    */
   public void ejbRemove() throws RemoteException {
      checkConnection();

      // Delete the appropriate row in table.
      try {
         // DELETE from DB.
         String sqlString = "DELETE FROM " + tableName + " WHERE accountid=?";
         PreparedStatement sqlStatement = jdbcConn.prepareStatement(sqlString);
         sqlStatement.setLong(1, accountId);

         // Execute query
         int updateResults = sqlStatement.executeUpdate();
         if (updateResults != 1) {
            // Either fewer than or more than one row was deleted
            throw(new RemoteException(getMsg("errRemove",Integer.toString(updateResults))));
         }
      }
      catch (Exception e) {
         // Something happened while deleting data
         throw(new RemoteException(getMsg("errDB"), e));
      }

      dropConnection();
   }
   /**
    * Stores the values of the persistent instance variables in the data source, overwriting
    * the values in the data source. 
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    */
   public void ejbStore () throws RemoteException {

      checkConnection();

      // Overwrite data in database using data in bean

      try {
         // UPDATE database
         String sqlString = "UPDATE " + tableName + " SET balance = ?, type = ? WHERE " 
                            + "accountid = ?";
         PreparedStatement sqlStatement = jdbcConn.prepareStatement(sqlString);
         sqlStatement.setFloat(1, balance);
         sqlStatement.setInt(2, type);
         sqlStatement.setLong(3, accountId);

         // Execute query
         int updateResults = sqlStatement.executeUpdate();
         if (updateResults != 1) {
            // Either fewer than or more than one row was updated
            throw(new RemoteException(getMsg("errStore", Integer.toString(updateResults))));
         }
      }
      catch (Exception e) {
         // Throw a resonable exception.
         throw(new RemoteException(getMsg("errDB"), e));
      }
   }
   /**
    * Returns the balance for the account.
    * @return float The balance in the account.
    */
   public float getBalance() {
      return balance; 
   }
   /**
    * Gets a connection to the data source used by the bean to store persistent data.
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    */
   private void makeConnection() throws RemoteException {

      try {
         // Open database connnection
         jdbcConn = DriverManager.getConnection(jdbcUrl);//, dbUser, dbPass);
      }
      catch (Exception e) {
         e.printStackTrace();
         throw new RemoteException(getMsg("dbConnectionFailed"), e);
      }
   }
   /**
    * Sets the account balance to the specified amount.
    * @param amount float The new account balance.
    */
   public void setBalance(float amount) {
      balance = amount; 
   }
   /**
    * Sets the entity context for the bean instance.
    * Also loads and registers the driver of the data source (DB2) 
    * used by each bean instance and gets other environment variables.    
    * The information is stored in the bean deployment descriptor.
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    */
   public void setEntityContext(EntityContext ctx) throws RemoteException {
      entityContext = ctx;

      try {
         getEnvProps();
         // Load the applet driver for DB2
         Class.forName(driverName);
      }
      catch (Exception e) {
         System.out.println(getMsg("accountBMBean"));
         e.printStackTrace();
         System.exit(1);
      }

   }
   /**
    * Subtracts the specified amount from the account balance and returns the new balance.
    * @param amount float The amount to subtract from the account balance.
    * @return float The new balance in the account after the method executes.
    * @exception com.ibm.ejs.doc.account.InsufficientFundsException The account contained 
    * less than the amount requested for subtraction.
    */
   public float subtract(float amount) throws InsufficientFundsException {
      if (balance < amount) {
         throw new InsufficientFundsException(getMsg("insufficientFunds"));
      }
      balance -= amount;
      return balance;
   }
   /**
    * Sets the entity context for the bean instance to null.
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    */
   public void unsetEntityContext() throws RemoteException {
      entityContext = null;
   }
   /**
     Private method to gather the environmental properties.
     The specific properties are:
     <ul>
     <li>URL for the database
     <li>Login account for this database, if applicable
     <li>Password for the account, if applicable
     </ul>
     
     @exception RemoteException Thrown when the request
     fails due to a lack of an EntityContext existing.
     */
   private void getEnvProps() throws RemoteException {

      if (entityContext == null)
         throw new RemoteException( getMsg("noContext") );

      if (entityContext.getEnvironment() == null)
         throw new RemoteException( getMsg("noEnvironment") );

      jdbcUrl = entityContext.getEnvironment().getProperty(DBURLProp);
      if (jdbcUrl == null)
         throw new RemoteException( getMsg("envNotSet", DBURLProp) );

      driverName = entityContext.getEnvironment().getProperty(DriverNameProp);
      if (driverName == null)
         throw new RemoteException( getMsg("envNotSet", DriverNameProp) );

      tableName = entityContext.getEnvironment().getProperty(TableNameProp);
      if (tableName == null)
         throw new RemoteException( getMsg("envNotSet", TableNameProp) );


      // it's OK if this property is not set
      DBLogin = entityContext.getEnvironment().getProperty(DBLoginProp);
      if (DBLogin == null || DBLogin.length() == 0)
         DBLogin = " ";        // be nice to Oracle :)

      // it's OK if this property is not set
      DBPassword = entityContext.getEnvironment().getProperty(DBPasswordProp);
      if (DBPassword == null || DBPassword.length() == 0)
         DBPassword = " ";     // be nice to Oracle :)
   }



   /** Provides NLS support by retrieving the message from the resource bundle. */
   private static String getMsg ( String id ) {
      if (msgs == null) {
         msgs = (ListResourceBundle)
                ResourceBundle.getBundle("com.ibm.ejs.doc.account.AccountResourceBundle");
      }
      return (String) msgs.handleGetObject( id );
   }

   /** Provides NLS support by retrieving the message from the resource bundle. */
   private static String getMsg ( String id, String arg1 ) {
      Object [] args = { arg1};
      return java.text.MessageFormat.format( getMsg( id ), args );
   }
}
