// 5648-C84, 5648-C83, (C) Copyright IBM Corporation, 1997, 2000
// All rights reserved. Licensed Materials Property of IBM
// Note to US Government users: Documentation related to restricted rights
// Use, duplication or disclosure is subject to restrictions set forth in GSA ADP Schedule with IBM Corp.
// This page may contain other proprietary notices and copyright information, the terms of which must be observed and followed.
//
// This program may be used, executed, copied, modified and distributed
// without royalty for the purpose of developing, using,
// marketing, or distributing.
//
// @(#) 1.4 src/ejb/java/com/ibm/ejs/doc/transfer/TransferBean.java, ejbdemo, boss 5/13/99 16:24:37 [5/25/99 15:39:22]
package com.ibm.ejs.doc.transfer;

/**
* The bean class for the Transfer session bean.
*/
import java.rmi.RemoteException;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.ListResourceBundle;
import javax.ejb.*;
import java.lang.*;
import javax.naming.*;
import com.ibm.ejs.doc.account.*;

// JNDI-related
import java.util.Properties;
import java.net.InetAddress;
import javax.naming.InitialContext;

public class TransferBean implements SessionBean {


   private static final String ORBCLASS_SYSPROP = "org.omg.CORBA.ORBClass";
   private static final String ORBCLASS_NAME = "com.ibm.CORBA.iiop.ORB";

   private static final String INITIAL_NAMING_FACTORY_SYSPROP = "javax.naming.Context.INITIAL_CONTEXT_FACTORY";
   private static final String PROVIDER_URL_SYSPROP = "javax.naming.Context.PROVIDER_URL";
   private static final String HOME_NAME_SYSPROP   = "JNDIName";

   // Variables for finding the home
   private String nameService = null;
   private String accountName = null;    
   private String providerURL = null;


   private SessionContext mySessionCtx = null;
   private InitialContext initialContext = null;
   private AccountHome accountHome = null;
   private Account fromAccount = null;
   private Account toAccount = null;
   /**
    * Notifies the bean instance that it is being activated.
    */
   public void ejbActivate() throws RemoteException {}
   /**
    * Notifies the bean instance that it is being created.
    */
   public void ejbCreate() throws RemoteException {
      // Get the initial context
      InitialContext ctx;

      try {

         // append to the system properties
         Properties env = System.getProperties();
         env.put( PROVIDER_URL_SYSPROP, getProviderURL());
         env.put( INITIAL_NAMING_FACTORY_SYSPROP, getNamingFactory());
         env.put( ORBCLASS_SYSPROP, ORBCLASS_NAME);

         ctx = new InitialContext( env );

      }
      catch (Exception e) {
         e.printStackTrace();
         throw new RemoteException(getMsg("errInitCtx")+e.getMessage());
      }

      // Lookup the home interface using the JNDI name
      try {

         ejbHome = ctx.lookup( getHomeName() );


         accountHome = (AccountHome) javax.rmi.PortableRemoteObject.narrow( ejbHome, com.ibm.ejs.doc.account.AccountHome.class );

      }
      catch (Exception e) {
         e.printStackTrace();
         throw new RemoteException(getMsg("errLookup")+e.getMessage());
      }

      if (ejbHome == null) {
         throw new RemoteException(getMsg("errLookup"));
      }
   }
   /**
    * Notifies the bean that it is being passivated.
    */
   public void ejbPassivate() throws RemoteException {}
   /**
    * Notifies the bean instance that it is being removed.
    */
   public void ejbRemove() throws RemoteException {}
   /**
    * Returns the balance of the account identified by the acctId parameter.
    * @return float The account balance.
    * @param acctId long The account Id for which a balance is returned.
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    * @exception javax.ejb.FinderException The specified account Id was not found.
    */
   public float getBalance(long acctId) throws FinderException, RemoteException {
      AccountKey key = new AccountKey(acctId);

      try {
         fromAccount = accountHome.findByPrimaryKey(key);
      }
      catch (FinderException ex) {
         throw new FinderException("Account " + acctId + " does not exist.");
      }
      return fromAccount.getBalance();
   }
   /**
    * Sets the session context for the bean instance.
    */
   public void setSessionContext(SessionContext ctx) throws RemoteException { 
      mySessionCtx = ctx; 
   }

   /**
    * Transfers the specified amount from the fromAcctId account to the toAcctId account.
    * @param fromAcctId long The account Id of the account from which the amount is transfered.
    * @param toAcctId long The account Id of the account to which the amount is transfered.
    * @param amount float The amount being transferred from the fromAcctId to the toAcctId.
    * @exception java.rmi.RemoteException A server-side exception caused the method to fail.
    * @exception javax.ejb.FinderException The specified account Id was not found.
    * @exception com.ibm.ejs.doc.account.InsufficientFundsException The account identified by
    * the toAcctId contained less than the amount requested for transfer.
    */
   public void transferFunds(long fromAcctId, long toAcctId, float amount) 
   throws RemoteException, InsufficientFundsException, FinderException {

      AccountKey fromKey = new AccountKey(fromAcctId);
      AccountKey toKey = new AccountKey(toAcctId);

      try {
         fromAccount = accountHome.findByPrimaryKey(fromKey);
      }
      catch (FinderException ex) {
         throw new FinderException("Account " + fromAcctId + " does not exist.");
      }

      try {
         toAccount = accountHome.findByPrimaryKey(toKey);
      }
      catch (FinderException ex) {
         throw new FinderException("Account " + toAcctId + " does not exist.");
      }

      try {
         toAccount.add(amount);
         fromAccount.subtract(amount);
      }
      catch (InsufficientFundsException ex) {
         mySessionCtx.setRollbackOnly();
         throw new InsufficientFundsException("Insufficient fund in " + fromAcctId);
      }
   }



   /** Helper method to get the initial naming factory. */
   private String getNamingFactory () throws RemoteException {
      //get the namingFactory from 
      //the EJB properties.

      if (mySessionCtx == null)
         throw new RemoteException( getMsg("noContext") );

      if (mySessionCtx.getEnvironment() == null)
         throw new RemoteException( getMsg("noEnvironment") );

      String nf = mySessionCtx.getEnvironment().getProperty(INITIAL_NAMING_FACTORY_SYSPROP);
      if (nf == null)
         throw new RemoteException( getMsg("envNotSet", INITIAL_NAMING_FACTORY_SYSPROP));

      return nf;
   }


   private String getProviderURL() throws RemoteException {
      //get the provider URL property either from 
      //the EJB properties or, if it isn't there
      //use "iiop://", which causes a default to the local host.
      if (mySessionCtx == null)
         throw new RemoteException( getMsg("noContext") );

      if (mySessionCtx.getEnvironment() == null)
         throw new RemoteException( getMsg("noEnvironment") );

      String pr = mySessionCtx.getEnvironment().getProperty(PROVIDER_URL_SYSPROP);
      if (pr == null)
         pr = "iiop://";

      return pr;
   }

   private String getHomeName() throws RemoteException {
      //get the home name from 
      //the EJB properties.

      if (mySessionCtx == null)
         throw new RemoteException( getMsg("noContext") );

      if (mySessionCtx.getEnvironment() == null)
         throw new RemoteException( getMsg("noEnvironment") );

      String hn = mySessionCtx.getEnvironment().getProperty(HOME_NAME_SYSPROP);
      if (hn == null)
         throw new RemoteException( getMsg("envNotSet", HOME_NAME_SYSPROP) );

      return hn;
   }


   /** Provides NLS support by retrieving the message from the resource bundle. */
   private static String getMsg ( String id ) {
      if (msgs == null) {
         msgs = (ListResourceBundle)
                ResourceBundle.getBundle("com.ibm.ejs.doc.account.AccountResourceBundle");
      }
      return (String) msgs.handleGetObject( id );
   }

   /** Provides NLS support by retrieving the message from the resource bundle. */
   private static String getMsg ( String id, String arg1 ) {
      Object [] args = { arg1};
      return java.text.MessageFormat.format( getMsg( id ), args );
   }

   // instance data
   private transient Object          ejbHome     = null;
   private transient org.omg.CORBA.ORB orb       = null;

   // static data
   private static transient ListResourceBundle msgs = null;


   private String    JNDIName             = null;  // "invalid"
   private String    initialNamingFactory = null;  // "invalid"


}
